
from dataclasses import dataclass
from textual.reactive import reactive
from textual.app import ComposeResult
from textual.widgets import Button, Label
from textual.containers import VerticalGroup, VerticalScroll

from blackwall.api.setropts import BaseSetroptsTraits, get_racf_options, update_racf_options
from blackwall.emoji import get_emoji
from blackwall.modals import generic_confirmation_modal
from blackwall.panels.traits_ui import generate_trait_inputs, set_traits_in_input, get_traits_from_input, toggle_inputs
from blackwall.panels.panel_mode import PanelMode

@dataclass
class SetroptsInfo:
    mode: PanelMode = PanelMode.read

class PanelSetroptsNotice(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Warning: this panel controls and displays RACF system options. It is not recommended to touch this unless you absolutely know what you are doing!",classes="setropts-warning")

class PanelSetroptsMode(VerticalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.read,recompose=True)

    def __init__(self, switch_action: str):
        super().__init__()
        self.switch_action = switch_action


    def compose(self) -> ComposeResult:
        if self.edit_mode is PanelMode.read:
            readable_mode = "read"
        elif self.edit_mode is PanelMode.edit:
            readable_mode = "edit"
        else:
            readable_mode = "read"
        
        yield Label(f"Mode: {readable_mode}",classes="setropts-mode-label")
        yield Button("Switch",tooltip="Toggle between read and edit mode",action="switch",classes="action-button")

    async def action_switch(self):
        await self.app.run_action(self.switch_action,default_namespace=self.parent)

class PanelSetroptsFields(VerticalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.read)
    base_traits: reactive[BaseSetroptsTraits] = reactive(BaseSetroptsTraits())

    def compose(self) -> ComposeResult:
        yield from generate_trait_inputs(prefix="base",traits_class=BaseSetroptsTraits)

    def watch_base_traits(self):
        set_traits_in_input(self,traits=self.base_traits,prefix="base")

    def watch_edit_mode(self):
        if self.edit_mode is PanelMode.read:
            toggle_inputs(self,prefix="base",traits=self.base_traits,disabled=True)
        elif self.edit_mode is PanelMode.edit:
            toggle_inputs(self,prefix="base",traits=self.base_traits,disabled=False)


class PanelSetroptsActionButtons(VerticalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.read)

    def __init__(self, save_action: str):
        super().__init__()
        self.save_action = save_action

    def compose(self) -> ComposeResult:
        yield Label("Attention: changing system settings can be dangerous!",classes="setropts-warning")
        if self.edit_mode is PanelMode.read:
            yield Button(f"{get_emoji("💾")} Save",id="save",action="save",variant="warning",classes="action-button",disabled=True)
        elif self.edit_mode is PanelMode.edit:
            yield Button(f"{get_emoji("💾")} Save",id="save",action="save",variant="warning",classes="action-button",disabled=False)

    async def action_save(self):
        await self.app.run_action(self.save_action,default_namespace=self.parent)

class PanelSetropts(VerticalScroll):
    setropts_info: reactive[SetroptsInfo] = reactive(SetroptsInfo())
    base_traits: reactive[BaseSetroptsTraits] = reactive(BaseSetroptsTraits())

    def watch_setropts_info(self, value: SetroptsInfo):
        mode_section = self.get_child_by_type(PanelSetroptsMode)
        mode_section = self.get_child_by_type(PanelSetroptsFields)
        #valid modes: edit and read
        mode_section.edit_mode = value.mode
  
    def on_mount(self) -> None:
        racf_options = get_racf_options()
        self.get_child_by_type(PanelSetroptsFields).base_traits = BaseSetroptsTraits.from_dict(prefix="base",source=racf_options["profile"]["base"])

    def compose(self) -> ComposeResult:
        yield PanelSetroptsNotice()
        yield PanelSetroptsMode(switch_action="switch")
        yield PanelSetroptsFields()
        yield PanelSetroptsActionButtons(save_action="save")

    def action_save_setropts_api(self) -> None:
        base_traits = get_traits_from_input(prefix="base",operator="alter",trait_cls=BaseSetroptsTraits,widget=self)
        message, return_code = update_racf_options(base=base_traits)
        if return_code == 0 or return_code == 4:
            self.notify(f"Updated system settings, return code: {return_code}",severity="warning")
        else:
            self.notify(f"Couldn't update system settings, return code: {return_code}",severity="error")

    def action_save(self) -> None:
        generic_confirmation_modal(self,modal_text="Are you abosulutely sure you want to change the RACF system options?",action_widget=self,confirm_action="save_setropts_api")

    def action_switch(self) -> None:
        if self.setropts_info.mode is PanelMode.read:
            self.setropts_info = SetroptsInfo(mode=PanelMode.edit) 
            self.query_exactly_one("#save",Button).disabled = False
            readable_mode = "edit"
        elif self.setropts_info.mode is PanelMode.edit:
            self.setropts_info = SetroptsInfo(mode=PanelMode.read) 
            readable_mode = "read"
            self.query_exactly_one("#save",Button).disabled = True
        else:
            readable_mode = "read"

        self.notify(f"Switched to {readable_mode}")




from dataclasses import dataclass

from textual.app import ComposeResult
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll
from textual.lazy import Lazy
from textual.reactive import reactive
from textual.widgets import Button, Collapsible, Input, Label, RadioButton, Select

from blackwall.api import user
from blackwall.emoji import get_emoji
from blackwall.modals import generic_confirmation_modal
from blackwall.notifications import send_notification
from blackwall.panels.panel_mode import PanelMode
from blackwall.regex import racf_id_regex

from ..traits_ui import (
    generate_trait_section,
    get_traits_from_input,
    set_traits_in_input,
)


class PanelUserInfo(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Creation date:",classes="date-labels")
        yield Input(id="base_create_date",disabled=True,classes="date-fields")
        yield Label("Last access date:",classes="date-labels")
        yield Input(id="base_last_access_date",disabled=True,classes="date-fields")
        #yield Label("Last access time:",classes="date-labels")
        #yield Input(id="last_acess_time",disabled=True,classes="date-fields")

class PanelUserName(HorizontalGroup):
    """Username and name components"""
    username: reactive[str] = reactive("")

    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    username_is_disabled = edit_mode is True

    def compose(self) -> ComposeResult:
        yield Label("Username*:")
        yield Input(max_length=8,restrict=racf_id_regex,id="username",classes="field-short-generic",tooltip="Username is what the user uses to log on with, this is required. While very few characters can be used at least 4 character long usernames are recommended to avoid collisions",disabled=self.username_is_disabled).data_bind(value=PanelUserName.username)
        yield Label("name:")
        yield Input(max_length=20,id="base_name",classes="name",tooltip="For personal users this is typically used for names i.e. Song So Mi, for system users it can be the name of the subsystem that it is used for")

class PanelUserOwnership(HorizontalGroup):
    """Component that contains ownership field and default group"""
    def compose(self) -> ComposeResult:
        yield Label("Owner:")
        yield Input(max_length=8,restrict=racf_id_regex,id="base_owner",classes="field-short-generic", tooltip="The group or user that owns this user profile. This is required in the RACF database")
        yield Label("Default group*:")
        yield Input(max_length=8,restrict=racf_id_regex,id="base_default_group",classes="field-short-generic", tooltip="All users must belong to a group in the RACF database")
        yield Label("Default group authority:")
        #yield Input(id="base_default_group_authority",classes="field-short-generic",max_length=8)
        yield Select([("USE", "USE"),("CREATE", "CREATE"),("CONNECT", "CONNECT"),("JOIN", "JOIN")],id="base_default_group_authority",value="USE",classes="uacc-select")

class PanelUserInstalldata(HorizontalGroup):
    """Component that contains install data field"""
    def compose(self) -> ComposeResult:
        yield Label("Installation data: ")
        yield Input(max_length=255,id="base_installation_data",classes="installation-data",tooltip="Installation data is an optional piece of data you can assign to a user. You can use installation data to describe whatever you want, such as department or what the user is for")

class PanelUserPassword(VerticalGroup):
    """Change/add password component"""
    def compose(self) -> ComposeResult:
        with Lazy(widget=Collapsible(title="Password")):
            yield Label("Passwords can only be 8 characters long")
            yield Label("New password:")
            yield Input(max_length=8,id="base_password",classes="password",password=True)

class PanelUserPassphrase(VerticalGroup):
    """Change/add passphrase component"""
    def compose(self) -> ComposeResult:
        with Lazy(widget=Collapsible(title="Passphrase")):
            yield Label("Passphrases need to be between 12 and 100 characaters long")
            yield Label("New passphrase:")
            yield Input(max_length=100,id="base_passphrase",classes="passphrase",password=True)
    
class PanelUserAttributes(VerticalGroup):
    """User attributes component"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="User attributes"):
            yield RadioButton("Special",id="base_special",tooltip="This is RACF's way of making a user admin. Special users can make other users special, making this a potentially dangerous option")
            yield RadioButton("Operations",id="base_operations",tooltip="This is a very dangerous attribute that allows you to bypass most security checks on the system, this should only be used during maintenance tasks and removed immediately afterwards")
            yield RadioButton("Auditor",id="base_auditor")

class PanelUserLevelAndCategory(VerticalGroup):
    """User attributes component"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="Security level and category"):
            yield Label("Security level:")
            yield Input(max_length=8,id="base_security_level",classes="field-short-generic")
            yield Label("Security category:")
            yield Input(max_length=8,id="base_security_category",classes="field-short-generic")
            yield Label("Security label:")
            yield Input(max_length=8,id="base_security_label",classes="field-short-generic")

class PanelUserDatasetsAndUACC(VerticalGroup):
    """User attributes component"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="Datasets and UACC"):
            yield Label("UACC:")
            yield Select([("NONE", "NONE"),("READ", "READ"),("EXECUTE", "EXECUTE"),("UPDATE", "UPDATE"),("CONTROL", "CONTROL"),("ALTER", "ALTER")],id="base_universal_access",value="NONE",classes="uacc-select")
            yield Label("Model dataset:")
            yield Input(max_length=255,id="base_model_data_set",classes="field-long-generic")
            yield RadioButton(label="Group dataset access",id="base_group_data_set_access",classes="generic-checkbox-medium")

class PanelUserSegments(VerticalGroup):
    """Component where the user can add segments such as the OMVS segment"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="User segments"):
            yield from generate_trait_section(title="TSO", prefix="tso", traits_class=user.TSOUserTraits)
            yield from generate_trait_section(title="OMVS (z/OS Unix)", prefix="omvs", traits_class=user.OMVSUserTraits)
            yield from generate_trait_section(title="Work attributes", prefix="workattr", traits_class=user.WorkattrUserTraits)
            yield from generate_trait_section(title="CICS", prefix="cics", traits_class=user.CICSUserTraits)
            yield from generate_trait_section(title="KERB", prefix="kerb", traits_class=user.KerbUserTraits)
            yield from generate_trait_section(title="Language", prefix="language", traits_class=user.LanguageUserTraits)
            yield from generate_trait_section(title="OPERPARM", prefix="operparm", traits_class=user.OperparmUserTraits)
            yield from generate_trait_section(title="OVM", prefix="ovm", traits_class=user.OvmUserTraits)
            yield from generate_trait_section(title="NDS", prefix="nds", traits_class=user.NDSUserTraits)
            yield from generate_trait_section(title="Netview", prefix="netview", traits_class=user.NetviewUserTraits)
            yield from generate_trait_section(title="MFA", prefix="mfa", traits_class=user.MfaUserTraits)
            yield from generate_trait_section(title="DCE", prefix="dce", traits_class=user.DCEUserTraits)
            yield from generate_trait_section(title="DFP", prefix="dfp", traits_class=user.DFPUserTraits)
            yield from generate_trait_section(title="EIM", prefix="eim", traits_class=user.EIMUserTraits)
            yield from generate_trait_section(title="Proxy", prefix="proxy", traits_class=user.ProxyUserTraits)
            yield from generate_trait_section(title="Lotus Notes", prefix="lnotes", traits_class=user.LnotesUserTraits)

class PanelUserActionButtons(HorizontalGroup):
    """Action buttons"""
    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    delete_is_disabled = edit_mode is not True

    def __init__(self, save_action: str, delete_action: str):
        super().__init__()
        self.save_action = save_action
        self.delete_action = delete_action

    def compose(self) -> ComposeResult:
        if self.edit_mode == PanelMode.create:
            yield Button("Create", tooltip="This will update the user, or create it if the user doesn't exist",action="save",classes="action-button",id="save")
        elif self.edit_mode == PanelMode.edit:
            yield Button(f"{get_emoji("💾")} Save", tooltip="This will update the user, or create it if the user doesn't exist",action="save",classes="action-button",id="save")
        yield Button("Delete", tooltip="This will delete the user permanently from the RACF database",id="delete",action="delete",variant="error",classes="action-button",disabled=self.delete_is_disabled)

    async def action_save(self):
        await self.app.run_action(self.save_action,default_namespace=self.parent)

    async def action_delete(self):
        await self.app.run_action(self.delete_action,default_namespace=self.parent)

@dataclass
class UserInfo:
    base_traits: user.BaseUserTraits | None = None
    tso_traits: user.TSOUserTraits | None = None
    omvs_traits: user.OMVSUserTraits | None = None
    cics_traits: user.CICSUserTraits | None = None
    kerb_traits: user.KerbUserTraits | None = None
    eim_traits: user.EIMUserTraits | None = None
    lang_traits: user.LanguageUserTraits | None = None
    dce_traits: user.DCEUserTraits | None = None
    dfp_traits: user.DFPUserTraits | None = None
    nds_traits: user.NDSUserTraits | None = None
    lnotes_traits: user.LnotesUserTraits | None = None
    mfa_traits: user.MfaUserTraits | None = None
    ovm_traits: user.OvmUserTraits | None = None
    proxy_traits: user.ProxyUserTraits | None = None
    workattr_traits: user.WorkattrUserTraits | None = None
    netview_traits: user.NetviewUserTraits | None = None
    operparm_traits: user.OperparmUserTraits | None = None

    mode: PanelMode = PanelMode.create
    username: str = ""


class PanelUser(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelUserInfo()
        yield PanelUserName()
        yield PanelUserOwnership()
        yield PanelUserInstalldata()
        yield PanelUserPassword()
        yield PanelUserPassphrase()
        yield PanelUserDatasetsAndUACC()
        yield PanelUserAttributes()
        yield PanelUserLevelAndCategory()
        yield PanelUserSegments()
        yield PanelUserActionButtons(save_action="save_user", delete_action="delete_user")
    
    user_info: reactive[UserInfo] = reactive(UserInfo())
    
    def on_mount(self) -> None:
        if user.user_exists(self.user_info.username):
            self.query_exactly_one("#username",Input).value = self.user_info.username
            if self.user_info.base_traits is not None:
                set_traits_in_input(self,traits=self.user_info.base_traits,prefix="base")
            
            if self.user_info.tso_traits is not None:
                set_traits_in_input(self,traits=self.user_info.tso_traits,prefix="tso")

            if self.user_info.omvs_traits is not None:
                set_traits_in_input(self,traits=self.user_info.omvs_traits,prefix="omvs")
                
            if self.user_info.cics_traits is not None:
                set_traits_in_input(self,traits=self.user_info.cics_traits,prefix="cics")
            
            if self.user_info.mfa_traits is not None:
                set_traits_in_input(self,traits=self.user_info.mfa_traits,prefix="mfa")

            if self.user_info.lang_traits is not None:
                set_traits_in_input(self,traits=self.user_info.lang_traits,prefix="language")
            
            if self.user_info.dce_traits is not None:
                set_traits_in_input(self,traits=self.user_info.dce_traits,prefix="dce")

            if self.user_info.dfp_traits is not None:
                set_traits_in_input(self,traits=self.user_info.dfp_traits,prefix="dfp")

            if self.user_info.netview_traits is not None:
                set_traits_in_input(self,traits=self.user_info.netview_traits,prefix="netview")

            if self.user_info.lnotes_traits is not None:
                set_traits_in_input(self,traits=self.user_info.lnotes_traits,prefix="lnotes")

            if self.user_info.ovm_traits is not None:
                set_traits_in_input(self,traits=self.user_info.ovm_traits,prefix="ovm")

            if self.user_info.nds_traits is not None:
                set_traits_in_input(self,traits=self.user_info.nds_traits,prefix="nds")

            if self.user_info.workattr_traits is not None:
                set_traits_in_input(self,traits=self.user_info.workattr_traits,prefix="workattr")

            if self.user_info.proxy_traits is not None:
                set_traits_in_input(self,traits=self.user_info.proxy_traits,prefix="proxy")

            if self.user_info.eim_traits is not None:
                set_traits_in_input(self,traits=self.user_info.eim_traits,prefix="eim")
            
            if self.user_info.operparm_traits is not None:
                set_traits_in_input(self,traits=self.user_info.operparm_traits,prefix="operparm")

            self.set_edit_mode()

    def set_edit_mode(self):
        #user_name_panel = self.get_child_by_type(PanelUserName)
        #user_name_panel.mode = PanelMode.edit
        self.query_exactly_one("#username",Input).disabled = True
        self.query_exactly_one("#delete",Button).disabled = False
        self.query_exactly_one("#save",Button).label = f"{get_emoji("💾")} Save"
        self.notify("Switched to edit mode",severity="information")

    def action_delete_user_api(self):
        username = self.get_child_by_type(PanelUserName).get_child_by_id("username",Input).value
        if user.user_exists(username=username):
            message, return_code = user.delete_user(username)
            
            if (return_code == 0):
                self.notify(f"User {username} deleted, return code: {return_code}",severity="warning")
            else:
                self.notify(f"{message}, return code: {return_code}",severity="error")

    def action_delete_user(self) -> None:
        username = self.get_child_by_type(PanelUserName).get_child_by_id("username",Input).value
        generic_confirmation_modal(self,modal_text=f"Are you sure you want to delete user {username}?",confirm_action="delete_user_api",action_widget=self)

    def action_save_user(self) -> None:
        username = self.get_child_by_type(PanelUserName).get_child_by_id("username",Input).value
        user_exists = user.user_exists(username=username)

        operator = "alter" if user_exists else "add"
        
        base_segment = get_traits_from_input(operator, self, prefix="base", trait_cls=user.BaseUserTraits)
        tso_segment = get_traits_from_input(operator, self, prefix="tso", trait_cls=user.TSOUserTraits)
        omvs_segment = get_traits_from_input(operator, self, prefix="omvs", trait_cls=user.OMVSUserTraits)
        cics_segment = get_traits_from_input(operator, self, prefix="cics", trait_cls=user.CICSUserTraits)
        workattr_segment = get_traits_from_input(operator, self, prefix="workattr", trait_cls=user.WorkattrUserTraits)
        language_segment = get_traits_from_input(operator, self, prefix="language", trait_cls=user.LanguageUserTraits)
        dfp_segment = get_traits_from_input(operator, self, prefix="dfp", trait_cls=user.DFPUserTraits)
        dce_segment = get_traits_from_input(operator, self, prefix="dce", trait_cls=user.DCEUserTraits)
        proxy_segment = get_traits_from_input(operator, self, prefix="proxy", trait_cls=user.ProxyUserTraits)
        operparm_segment = get_traits_from_input(operator, self, prefix="operparm", trait_cls=user.OperparmUserTraits)
        ovm_segment = get_traits_from_input(operator, self, prefix="ovm", trait_cls=user.OvmUserTraits)
        eim_segment = get_traits_from_input(operator, self, prefix="eim", trait_cls=user.EIMUserTraits)
        nds_segment = get_traits_from_input(operator, self, prefix="nds", trait_cls=user.NDSUserTraits)
        lnotes_segment = get_traits_from_input(operator, self, prefix="lnotes", trait_cls=user.LnotesUserTraits)
        mfa_segment = get_traits_from_input(operator, self, prefix="mfa", trait_cls=user.MfaUserTraits)
        netview_segment = get_traits_from_input(operator, self, prefix="netview", trait_cls=user.NetviewUserTraits)
        result = user.update_user(
            username=username,
            create=not user_exists,
            base=base_segment,
            tso=tso_segment,
            omvs=omvs_segment,
            cics=cics_segment,
            workattr=workattr_segment,
            language=language_segment,
            dfp=dfp_segment,
            dce=dce_segment,
            proxy=proxy_segment,
            operparm=operparm_segment,
            ovm=ovm_segment,
            eim=eim_segment,
            nds=nds_segment,
            lnotes=lnotes_segment,
            mfa=mfa_segment,
            netview=netview_segment,
        )

        if not user_exists:
            if (result == 0 or result == 4):
                self.notify(f"User {username} created, return code: {result}",severity="information")
                self.set_edit_mode()
            else:
                send_notification(self,message=f"Unable to create user, return code: {result}",severity="error")
        else:
            if result == 0:
                self.notify(f"User {username} updated, return code: {result}",severity="information")
            else:
                send_notification(self,message=f"Unable to update user, return code: {result}",severity="error")



#Permit API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

import importlib.util
from dataclasses import dataclass, field

from .traits_base import TraitsBase

#Checks if RACFU can be imported
racfu_enabled = importlib.util.find_spec('racfu')

if racfu_enabled:
    from racfu import racfu  # type: ignore
else:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")

@dataclass
class BasePermitTraits(TraitsBase):
    access: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    model_profile_class: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    model_profile: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    model_profile_generic: bool | None = field(default=None,metadata={"allowed_in": {"alter"}})
    model_profile_volume: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    reset: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_partner_lu_name: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_console: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_jes: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_program: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_servauth: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_sms: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_service: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_system: str | None = field(default=None,metadata={"allowed_in": {"alter"}})
    when_terminal: str | None = field(default=None,metadata={"allowed_in": {"alter"}})

def update_dataset_permit(dataset: str, racf_id: str, base: BasePermitTraits) -> int:
    """Creates or updates a dataset permit"""
    if racfu_enabled:
        traits = base.to_traits(prefix="base")

        result = racfu({"operation": "alter", "admin_type": "permission", "data_set": dataset.upper(), "auth_id": racf_id.upper(), "traits":  traits})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return 8

def delete_dataset_permit(dataset: str, racf_id: str) -> int:
    """Deletes a dataset permit"""
    if racfu_enabled:
        result = racfu({"operation": "delete", "admin_type": "permission", "data_set": dataset.upper(), "auth_id": racf_id.upper()})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return 8

def update_resource_permit(profile: str ,class_name: str, racf_id: str, base: BasePermitTraits) -> int:
    """Creates or updates a general resource profile permit"""
    if racfu_enabled:
        traits = base.to_traits(prefix="base")

        result = racfu({"operation": "alter", "admin_type": "permission", "profile_name": profile.upper(), "class_name": class_name.upper(), "auth_id": racf_id.upper(), "traits":  traits})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return 8

def delete_resource_permit(profile: str, class_name: str, racf_id: str) -> int:
    """Deletes a general resource profile permit"""
    if racfu_enabled:
        result = racfu({"operation": "delete", "admin_type": "permission", "profile_name": profile.upper(), "class_name": class_name.upper(), "auth_id": racf_id.upper()})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return 8




from textual.notifications import SeverityLevel

from blackwall.modals import generic_warning_modal
from blackwall.settings import get_user_setting


def send_notification(self,message: str, severity: SeverityLevel):
    if get_user_setting(section="notifications",setting="use_modal"):
        generic_warning_modal(self,modal_text=message)
    else:
        self.notify(message,severity=severity)



import importlib.util
from importlib.resources import files
from pathlib import Path

from textual.app import ComposeResult
from textual.containers import Grid, VerticalGroup, VerticalScroll
from textual.widgets import Button, Label, Markdown

from blackwall.messages import OpenTab
from blackwall.panels.analysis.analysis import PanelAnalysis
from blackwall.panels.dataset.dataset import PanelDataset
from blackwall.panels.group.group import PanelGroup
from blackwall.panels.resource.resource import PanelResource
from blackwall.panels.search.search import PanelSearch
from blackwall.panels.setropts.setropts import PanelSetropts
from blackwall.panels.users.user import PanelUser
from blackwall.settings import get_site_setting, get_user_setting

message = files('blackwall.panels.welcome').joinpath('welcome_message.md').read_text()

logo_allowed = get_user_setting(section="display",setting="logo")
site_logo_path = get_site_setting(section="welcome",setting="logo_path")

if Path(f"{site_logo_path}").exists():
    logo_path = f"{site_logo_path}"
else:
    logo_path = "OMP_CBTTape_original-color.png"

class PanelWelcomeLogo(VerticalGroup):
    def __init__(self, logo_path: Path | str):
        super().__init__()
        self.logo_path = logo_path

    def compose(self) -> ComposeResult:
        if logo_allowed is not False:
            textual_image_enabled = importlib.util.find_spec('textual_image')
            if textual_image_enabled:
                from textual_image.widget import SixelImage
                image = Path(f'{logo_path}')
                yield SixelImage(image, classes="logo-image")

class PanelWelcomeMessage(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Markdown(message,classes="welcome-message")

class PanelWelcomeActions(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Try out the program:",classes="welcome-suggestion-header")
        yield Button("Search RACF", classes="welcome-suggestion-button",action="search")
        yield Button("Create user", classes="welcome-suggestion-button",action="create_user")
        yield Button("Create group", classes="welcome-suggestion-button",action="create_group")
        yield Button("Create dataset profile", classes="welcome-suggestion-button",action="create_dataset")
        yield Button("Create general resource profile", classes="welcome-suggestion-button",action="create_resource")
        yield Button("View system options", classes="welcome-suggestion-button",action="view_options")
        #yield Button("Analyse system health", classes="welcome-suggestion-button",action="create_analysis")

    async def action_search(self):
        self.post_message(OpenTab(title="Search",content=PanelSearch()))

    async def action_create_user(self):
        self.post_message(OpenTab(title="Create user",content=PanelUser()))

    async def action_create_group(self):
        self.post_message(OpenTab(title="Create group",content=PanelGroup()))

    async def action_create_dataset(self):
        self.post_message(OpenTab(title="Create dataset profile",content=PanelDataset()))

    async def action_create_resource(self):
        self.post_message(OpenTab(title="Create resource profile",content=PanelResource()))

    async def action_view_options(self):
        self.post_message(OpenTab(title="RACF options",content=PanelSetropts()))

    async def action_create_analysis(self):
        self.post_message(OpenTab(title="Health check",content=PanelAnalysis()))

class PanelWelcomeContent(Grid):
    def compose(self) -> ComposeResult:
        yield PanelWelcomeMessage()
        yield PanelWelcomeLogo(logo_path=logo_path)
        yield PanelWelcomeActions()

class PanelWelcome(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelWelcomeContent()


#Group API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

import importlib.util
from dataclasses import dataclass, field

from .traits_base import TraitsBase

#Checks if RACFU can be imported
racfu_enabled = importlib.util.find_spec('racfu')

if racfu_enabled:
    from racfu import racfu  # type: ignore
else:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")       

@dataclass
class BaseGroupTraits(TraitsBase):
    owner: str | None = field(default=None,metadata={"label": "Owner", "allowed_in": {"add","alter","extract"}})
    installation_data: str | None = field(default=None,metadata={"label": "Intallation data", "allowed_in": {"add","alter","extract"}})
    data_set_model: str | None = field(default=None,metadata={"label": "Dataset model", "allowed_in": {"add","alter","extract"}})
    superior_group: str | None = field(default=None,metadata={"label": "Superior group","allowed_in": {"add","alter","extract"}})
    terminal_universal_access: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})
    universal: str | None = field(default=None,metadata={"allowed_in": {"add","extract"}})
    subgroups: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    subgroup: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    connected_users: list[str] | None = field(default=None,metadata={"allowed_in": {"extract"}})
    connected_user_authority: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    connected_userid: str | None = field(default=None,metadata={"allowed_in": {"extract"}})
    create_date: str | None = field(default=None,metadata={"allowed_in": {"extract"}})

@dataclass
class DFPGroupTraits(TraitsBase):
    data_application: str | None = field(default=None,metadata={"label": "Data application", "allowed_in": {"add","alter","extract"}, "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    data_class: str | None = field(default=None,metadata={"label": "Data class", "allowed_in": {"add","alter","extract"}, "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    management_class: str | None = field(default=None,metadata={"label": "Management class", "allowed_in": {"add","alter","extract"}, "input_args": {"max_length": 8,"classes": "field-short-generic"}})
    storage_class: str | None = field(default=None,metadata={"label": "Storage class", "allowed_in": {"add","alter","extract"}, "input_args": {"max_length": 8,"classes": "field-short-generic"}})

@dataclass
class OMVSGroupTraits(TraitsBase):
    auto_gid: str | None = field(default=None,metadata={"allowed_in": {"add","alter"},"invalid_values": {False}})
    gid: int | None = field(default=None,metadata={"label": "GID", "allowed_in": {"add","alter","extract"}})
    shared: str | None = field(default=None,metadata={"allowed_in": {"add","alter"},"invalid_values": {False}})

@dataclass
class OVMGroupTraits(TraitsBase):
    gid: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})

@dataclass
class TMEGroupTraits(TraitsBase):
    roles: str | None = field(default=None,metadata={"allowed_in": {"add","alter","extract"}})

#Group functions
def group_exists(group: str) -> bool:
    """Checks if a group exists, returns true or false"""
    if racfu_enabled:
        result = racfu({"operation": "extract", "admin_type": "group", "profile_name": group.upper()})
        return result.result["return_codes"]["racf_return_code"] == 0
    else:
        return False

def get_group(group: str) -> dict:
    """Doesn't handle group profiles that don't exist, recommend using group_exists() first"""
    if racfu_enabled:
        result = racfu({"operation": "extract", "admin_type": "group", "profile_name": group.upper()})
        return result.result
    else:
        return {}

def get_group_connections(group: str):
    """Get information on group connections"""
    pass

def update_group(group: str,create: bool, base: BaseGroupTraits, tme: TMEGroupTraits | None = None, omvs: OMVSGroupTraits | None = None, dfp: DFPGroupTraits | None = None, ovm: OVMGroupTraits | None = None):
    if racfu_enabled:
        """Creates or updates an existing group"""
        traits = base.to_traits(prefix="base")
        
        if tme is not None:
            traits.update(tme.to_traits("tme"))

        if dfp is not None:
            traits.update(dfp.to_traits("dfp"))

        if omvs is not None:
            traits.update(omvs.to_traits("omvs"))

        if ovm is not None:
            traits.update(ovm.to_traits("ovm"))

        operation = "add" if create else "alter"
        
        result = racfu(
            {
                "operation": operation, 
                "admin_type": "group", 
                "profile_name": group.upper(),
                "traits":  traits,
            },
        )
        return result.result["return_codes"]["racf_return_code"]

def delete_group(group: str) -> tuple[str, int]:
    if racfu_enabled:
        """Deletes a group"""
        result = racfu(
            {
                "operation": "delete", 
                "admin_type": "group", 
                "profile_name": group.upper(),
            },
        )
        #TODO add error message
        return "", result.result["return_codes"]["racf_return_code"]
    else:
        return "RACFu can't be found", 8